@@ -1,4 +1,4 @@
-// $Id: gdbdriver.cpp,v 1.55 2005/11/20 21:31:33 jsixt Exp $
+// $Id$
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -235,6 +235,11 @@ bool GdbDriver::startup(QString cmdStr)
 	"set confirm off\n"
 	"set print static-members off\n"
 	"set print asm-demangle on\n"
+	/*
+	 * Don't assume that program functions invoked from a watch expression
+	 * always succeed.
+	 */
+	"set unwindonsignal on\n"
 	/*
 	 * Write a short macro that prints all locals: local variables and
 	 * function arguments.
@@ -647,10 +652,22 @@ static bool isErrorExpr(const char* output)
 /**
  * Returns true if the output is an error message. If wantErrorValue is
  * true, a new VarTree object is created and filled with the error message.
+ * If there are warnings, they are skipped and output points past the warnings
+ * on return (even if there \e are errors).
  */
-static bool parseErrorMessage(const char* output,
+static bool parseErrorMessage(const char*& output,
 			      VarTree*& variable, bool wantErrorValue)
 {
+    // skip warnings
+    while (strncmp(output, "warning:", 8) == 0)
+    {
+	char* end = strchr(output+8, '\n');
+	if (end == 0)
+	    output += strlen(output);
+	else
+	    output = end+1;
+    }
+
     if (isErrorExpr(output))
     {
 	if (wantErrorValue) {
@@ -695,12 +712,11 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
      * Parse off white space. gdb sometimes prints white space first if the
      * printed array leaded to an error.
      */
-    const char* p = output;
-    while (isspace(*p))
-	p++;
+    while (isspace(*output))
+	output++;
 
     // special case: empty string (0 repetitions)
-    if (strncmp(p, "Invalid number 0 of repetitions", 31) == 0)
+    if (strncmp(output, "Invalid number 0 of repetitions", 31) == 0)
     {
 	variable = new VarTree(QString(), VarTree::NKplain);
 	variable->m_value = "\"\"";
@@ -708,12 +724,13 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
     }
 
     // check for error conditions
-    if (parseErrorMessage(p, variable, wantErrorValue))
+    if (parseErrorMessage(output, variable, wantErrorValue))
 	return variable;
 
     // parse the array
 
     // find '='
+    const char* p = output;
     p = strchr(p, '=');
     if (p == 0) {
 	goto error;
@@ -906,6 +923,49 @@ static void skipNested(const char*& s, char opening, char closing)
     s = p;
 }
 
+/**
+ * This function skips text that is delimited by nested angle bracktes, '<>'.
+ * A complication arises because the delimited text can contain the names of
+ * operator<<, operator>>, operator<, and operator>, which have to be treated
+ * specially so that they do not count towards the nesting of '<>'.
+ * This function assumes that the delimited text does not contain strings.
+ */
+static void skipNestedAngles(const char*& s)
+{
+    const char* p = s;
+
+    int nest = 1;
+    p++;		// skip the initial '<'
+    while (*p && nest > 0)
+    {
+	// Below we can check for p-s >= 9 instead of 8 because
+	// *s is '<' and cannot be part of "operator".
+	if (*p == '<')
+	{
+	    if (p-s >= 9 && strncmp(p-8, "operator", 8) == 0) {
+		if (p[1] == '<')
+		    p++;
+	    } else {
+		nest++;
+	    }
+	}
+	else if (*p == '>')
+	{
+	    if (p-s >= 9 && strncmp(p-8, "operator", 8) == 0) {
+		if (p[1] == '>')
+		    p++;
+	    } else {
+		nest--;
+	    }
+	}
+	p++;
+    }
+    if (nest != 0) {
+	TRACE(QString().sprintf("parse error: mismatching <> at %-20.20s", s));
+    }
+    s = p;
+}
+
 /**
  * Find the end of line that is not inside braces
  */
@@ -1028,7 +1088,7 @@ static bool parseName(const char*& s, QString& name, VarTree::NameKind& kind)
     //  <string<a,b<c>,7> >
 
     if (*p == '<') {
-	skipNested(p, '<', '>');
+	skipNestedAngles(p);
 	name = FROM_LATIN1(s, p - s);
 	kind = VarTree::NKtype;
     }
@@ -1197,7 +1257,7 @@ repeat:
 	    checkMultiPart = true;
 	} else if (*p == '<') {
 	    // e.g. <optimized out>
-	    skipNested(p, '<', '>');
+	    skipNestedAngles(p);
 	} else if (*p == '"' || *p == '\'') {
 	    // character may have multipart: '\000' <repeats 11 times>
 	    checkMultiPart = *p == '\'';
@@ -1256,7 +1316,7 @@ repeat:
 		// if this value is part of an array, it might be followed
 		// by <repeats 15 times>, which we don't skip here
 		if (strncmp(p, "<repeats ", 9) != 0)
-		    skipNested(p, '<', '>');
+		    skipNestedAngles(p);
 	    }
 	    if (p != start) {
 		// there is always a blank before the string,
@@ -1466,8 +1526,18 @@ static void parseFrameInfo(const char*& s, QString& func,
     while (*p != '\0')
     {
 	if (*p == '<') {
-	    // skip template parameter list
-	    skipNested(p, '<', '>');
+	    // check for operator<< and operator<
+	    if (p-start >= 8 && strncmp(p-8, "operator", 8) == 0)
+	    {
+		p++;
+		if (*p == '<')
+		    p++;
+	    }
+	    else
+	    {
+		// skip template parameter list
+		skipNestedAngles(p);
+	    }
 	} else if (*p == '(') {
 	    if (strncmp(p, "(anonymous namespace)", 21) != 0)
 		break;	// parameter list found
@@ -2356,7 +2426,7 @@ void GdbDriver::parseDisassemble(const char* output, QList<DisassembledCode>& co
 	while (p != end && *p != '<')
 	    p++;
 	if (*p == '<')
-	    skipNested(p, '<', '>');
+	    skipNestedAngles(p);
 	if (*p == ':')
 	    p++;
 
